// Dodati fajlove koje ste napravili
#include <fstream>
#include <iostream>
#include <string>
using namespace std;

/*
	Moguci izlaz:
		BG 000-AA

	Poeni:
		2
*/
void test_0() {
	fstream file("test_0.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	file << *(db.create("Beograd")) << endl;

	file.close();
}

/*
	Moguci izlaz:
		NS 000-AA
		NI 000-AA
		KG 000-AA
		SU 000-AA

	Poeni:
		2
*/
void test_1() {
	fstream file("test_1.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	file << *(db.create("Novi Sad")) << endl;
	file << *(db.create("Nis")) << endl;
	file << *(db.create("Kragujevac")) << endl;
	file << *(db.create("Subotica")) << endl;

	file.close();
}

/*
	Moguci izlaz:
		CA 000-AA
		CA 000-AB
		CA 000-AC
		CA 000-AD
		CA 000-AE

	Poeni:
		3
*/
void test_2() {
	fstream file("test_2.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	for(int i = 1; i <= 5; i++)
		file << *(db.create("Cacak")) << endl;

	file.close();
}

/*
	Moguci izlaz:
		VA 000-AA
		VA 000-AB
		...
		VA 999-ZY
		VA 999-ZZ
		Nije moguce kreirati novu tablicu.

	Poeni:
		3
*/
void test_3() {
	fstream file("test_3.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	try {
		for (int i = 1; i <= 676001; i++)
			file << *(db.create("Valjevo")) << endl;
	}
	catch (exception&) {
		file << "Nije moguce kreirati novu tablicu." << endl;
	}

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 4
		---------	---------
		---------	---------
		Broj vozila na ulazu: 0

	Poeni:
		2
*/
void test_4() {
	fstream file("test_4.txt", ios::out);

	ParkingLot parking_lot(2, 2);

	file << parking_lot << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 2
		NG 000-AA	PE 000-AA
		---------	---------
		Broj vozila na ulazu: 0

	Poeni:
		2
*/
void test_5() {
	fstream file("test_5.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	ParkingLot parking_lot(2, 2);

	Vehicle vehicle_1(db.create("Negotin"));
	Vehicle vehicle_2(db.create("Pec"));

	vehicle_1.requestEntrance(&parking_lot);
	vehicle_2.requestEntrance(&parking_lot);

	file << parking_lot << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 0
		AL 000-AA	
		AR 000-AA	
		Broj vozila na ulazu: 1
		VS 000-AA

		Parked
		Parked
		Waiting

	Poeni:
		2
*/
void test_6() {
	fstream file("test_6.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	ParkingLot parking_lot(2, 1);

	Vehicle vehicle_1(db.create("Aleksinac"));
	Vehicle vehicle_2(db.create("Arandjelovac"));
	Vehicle vehicle_3(db.create("Vrbas"));

	vehicle_1.requestEntrance(&parking_lot);
	vehicle_2.requestEntrance(&parking_lot);
	vehicle_3.requestEntrance(&parking_lot);

	file << parking_lot << endl;
	file << vehicle_1.getStateName() << endl;
	file << vehicle_2.getStateName() << endl;
	file << vehicle_3.getStateName() << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 1
		VL 000-AA	---------	
		BU 000-AA	GL 000-AA	
		Broj vozila na ulazu: 0

		Parked
		Driving
		Parked
		Parked

	Poeni:
		3
*/
void test_7() {
	fstream file("test_7.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	ParkingLot parking_lot(2, 2);

	Vehicle vehicle_1(db.create("Vlasotince"));
	Vehicle vehicle_2(db.create("Bogatic"));
	Vehicle vehicle_3(db.create("Bujanovac"));
	Vehicle vehicle_4(db.create("Gnjilane"));

	vehicle_1.requestEntrance(&parking_lot);
	vehicle_2.requestEntrance(&parking_lot);
	vehicle_3.requestEntrance(&parking_lot);
	vehicle_4.requestEntrance(&parking_lot);

	vehicle_2.requestExit(&parking_lot);

	file << parking_lot << endl;
	file << vehicle_1.getStateName() << endl;
	file << vehicle_2.getStateName() << endl;
	file << vehicle_3.getStateName() << endl;
	file << vehicle_4.getStateName() << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 0
		SI 000-AA	UE 000-AA	
		UR 000-AA	UB 000-AA	
		Broj vozila na ulazu: 1
		LU 000-AA

		Parked
		Driving
		Parked
		Parked
		Parked
		Waiting

	Poeni:
		3
*/
void test_8() {
	fstream file("test_8.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	ParkingLot parking_lot(2, 2);

	Vehicle vehicle_1(db.create("Sid"));
	Vehicle vehicle_2(db.create("Sabac"));
	Vehicle vehicle_3(db.create("Urosevac"));
	Vehicle vehicle_4(db.create("Ub"));
	Vehicle vehicle_5(db.create("Uzice"));
	Vehicle vehicle_6(db.create("Lucani"));

	vehicle_1.requestEntrance(&parking_lot);
	vehicle_2.requestEntrance(&parking_lot);
	vehicle_3.requestEntrance(&parking_lot);
	vehicle_4.requestEntrance(&parking_lot);
	vehicle_5.requestEntrance(&parking_lot);
	vehicle_6.requestEntrance(&parking_lot);

	vehicle_2.requestExit(&parking_lot);

	file << parking_lot << endl;
	file << vehicle_1.getStateName() << endl;
	file << vehicle_2.getStateName() << endl;
	file << vehicle_3.getStateName() << endl;
	file << vehicle_4.getStateName() << endl;
	file << vehicle_5.getStateName() << endl;
	file << vehicle_6.getStateName() << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 0
		CU 000-AA	TO 000-AA	
		SO 000-AA	TT 000-AA	
		Broj vozila na ulazu: 1
		TS 000-AA

		Parked
		Waiting
		Parked
		Parked
		Parked

	Poeni:
		3
*/
void test_9() {
	fstream file("test_9.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	ParkingLot parking_lot(2, 2);

	Vehicle vehicle_1(db.create("Cuprija"));
	Vehicle vehicle_2(db.create("Trstenik"));
	Vehicle vehicle_3(db.create("Topola"));
	Vehicle vehicle_4(db.create("Tutin"));
	Vehicle vehicle_5(db.create("Sombor"));

	vehicle_1.requestEntrance(&parking_lot);
	vehicle_2.requestEntrance(&parking_lot);
	vehicle_3.requestEntrance(&parking_lot);
	vehicle_4.requestEntrance(&parking_lot);

	vehicle_2.requestExit(&parking_lot);
	vehicle_3.requestExit(&parking_lot);

	vehicle_3.requestEntrance(&parking_lot);
	vehicle_5.requestEntrance(&parking_lot);
	vehicle_2.requestEntrance(&parking_lot);

	file << parking_lot << endl;
	file << vehicle_1.getStateName() << endl;
	file << vehicle_2.getStateName() << endl;
	file << vehicle_3.getStateName() << endl;
	file << vehicle_4.getStateName() << endl;
	file << vehicle_5.getStateName() << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 0
		RU 000-AA	RA 000-AA	
		Broj vozila na ulazu: 0

		Parked
		Parked
		Driving

	Poeni:
		3
*/
void test_10() {
	fstream file("test_10.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	ParkingLot parking_lot(1, 2);

	Vehicle vehicle_1(db.create("Ruma"));
	Vehicle vehicle_2(db.create("Raska"));
	Vehicle vehicle_3(db.create("Prijepolje"));

	vehicle_1.requestEntrance(&parking_lot);
	vehicle_2.requestEntrance(&parking_lot);
	vehicle_3.requestEntrance(&parking_lot);
	
	vehicle_3.requestExit(&parking_lot);

	file << parking_lot << endl;
	file << vehicle_1.getStateName() << endl;
	file << vehicle_2.getStateName() << endl;
	file << vehicle_3.getStateName() << endl;

	file.close();
}

/*
	Moguci izlaz:
		Vozilo je vec na parkingu ili ceka na ulazu.
		Vozilo nije na parkingu.

	Poeni:
		2
*/
void test_11() {
	fstream file("test_11.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	ParkingLot parking_lot(1, 2);

	Vehicle vehicle_1(db.create("Pec"));

	vehicle_1.requestEntrance(&parking_lot);
	try {
		vehicle_1.requestEntrance(&parking_lot);
	}
	catch (exception&) {
		file << "Vozilo je vec na parkingu ili ceka na ulazu." << endl;
	}

	vehicle_1.requestExit(&parking_lot);
	try {
		vehicle_1.requestExit(&parking_lot);
	}
	catch (exception&) {
		file << "Vozilo nije na parkingu.";
	}

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 0
		KO 000-AA(0)	
		KI 000-AA(0)	
		Broj vozila na ulazu: 0

	Poeni:
		1
*/
void test_12() {
	fstream file("test_12.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	TimeLimitedParkingLot parking_lot(2, 1, 3);

	Vehicle vehicle_1(db.create("Kovin"));
	Vehicle vehicle_2(db.create("Kikinda"));

	vehicle_1.requestEntrance(&parking_lot);
	vehicle_2.requestEntrance(&parking_lot);

	file << parking_lot << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 0
		NV 000-AA(1)	
		NP 000-AA(0)	
		Broj vozila na ulazu: 0

	Poeni:
		1
*/
void test_13() {
	fstream file("test_13.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	TimeLimitedParkingLot parking_lot(2, 1, 3);

	Vehicle vehicle_1(db.create("Nova Varos"));
	Vehicle vehicle_2(db.create("Novi Pazar"));

	vehicle_1.requestEntrance(&parking_lot);

	parking_lot++;
	vehicle_2.requestEntrance(&parking_lot);

	file << parking_lot << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 1
		---------	
		SV 000-AA(2)	
		Broj vozila na ulazu: 0

	Poeni:
		1
*/
void test_14() {
	fstream file("test_14.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	TimeLimitedParkingLot parking_lot(2, 1, 3);

	Vehicle vehicle_1(db.create("Smederevo"));
	Vehicle vehicle_2(db.create("Svilajnac"));

	vehicle_1.requestEntrance(&parking_lot);

	parking_lot++;
	vehicle_2.requestEntrance(&parking_lot);

	parking_lot++;
	parking_lot++;

	file << parking_lot << endl;

	file.close();
}

/*
	Moguci izlaz:
		Broj slobodnih mesta: 0
		LB 000-AA(2)	
		LE 000-AA(0)	
		Broj vozila na ulazu: 0

	Poeni:
		2
*/
void test_15() {
	fstream file("test_15.txt", ios::out);

	VRPDatabase& db = VRPDatabase::getInstance();

	TimeLimitedParkingLot parking_lot(2, 1, 3);

	Vehicle vehicle_1(db.create("Loznica"));
	Vehicle vehicle_2(db.create("Leskovac"));
	Vehicle vehicle_3(db.create("Lebane"));

	vehicle_1.requestEntrance(&parking_lot);

	parking_lot++;
	vehicle_2.requestEntrance(&parking_lot);
	vehicle_3.requestEntrance(&parking_lot);

	parking_lot++;
	parking_lot++;

	file << parking_lot << endl;

	file.close();
}

int main() {

	//test_0();
	//test_1();
	//test_2();
	//test_3();
	//test_4();
	//test_5();
	//test_6();
	//test_7();
	//test_8();
	//test_9();
	//test_10();
	//test_11();
	//test_12();
	//test_13();
	//test_14();
	//test_15();

	return 0;
}